@@ -11,8 +11,20 @@
 #    include <sys/ioctl.h>
 #    include <netinet/in.h>
 #    include <arpa/inet.h>
+#if defined(__sun)
+#include <unistd.h>
+#include <stropts.h>
+#include <sys/sockio.h>
+#endif
 #  endif /* HAVE_SOCKET_IOCTLS */
 
+/* For logical interfaces support we convert all names to same name prefixed with l */
+#if HAVE_SIOCGLIFNUM
+#define CNAME(x) l##x
+#else
+#define CNAME(x) x
+#endif
+
 #if HAVE_NET_IF_DL_H
 #  include <net/if_dl.h>
 #endif
@@ -109,6 +121,11 @@ static int af_to_len(int af)
 }
 
 #define SA_LEN(sa)      af_to_len(sa->sa_family)
+#if HAVE_SIOCGLIFNUM
+#define SS_LEN(sa)      af_to_len(sa->ss_family)
+#else
+#define SS_LEN(sa)      SA_LEN(sa)
+#endif
 #else
 #define SA_LEN(sa)      sa->sa_len
 #endif /* !HAVE_SOCKADDR_SA_LEN */
@@ -564,18 +581,18 @@ ifaddrs (PyObject *self, PyObject *args)
     return NULL;
   }
 
-  struct ifreq ifr;
+  struct CNAME(ifreq) ifr;
   PyObject *addr = NULL, *netmask = NULL, *braddr = NULL, *dstaddr = NULL;
   int is_p2p = FALSE;
   char buffer[256];
 
-  strncpy (ifr.ifr_name, ifname, IFNAMSIZ);
+  strncpy (ifr.CNAME(ifr_name), ifname, IFNAMSIZ);
 
 #if HAVE_SIOCGIFHWADDR
   if (ioctl (sock, SIOCGIFHWADDR, &ifr) == 0) {
     found = TRUE;
 
-    if (string_from_sockaddr (ifr->ifr_addr, buffer, sizeof (buffer)) == 0) {
+    if (string_from_sockaddr (ifr->CNAME(ifr_addr), buffer, sizeof (buffer)) == 0) {
       PyObject *hwaddr = PyString_FromString (buffer);
       PyObject *dict = PyDict_New ();
       PyObject *list = PyList_New (1);
@@ -604,46 +621,66 @@ ifaddrs (PyObject *self, PyObject *args)
 #endif
 
 #if HAVE_SIOCGIFADDR
+#if HAVE_SIOCGLIFNUM
+  if (ioctl (sock, SIOCGLIFADDR, &ifr) == 0) {
+#else
   if (ioctl (sock, SIOCGIFADDR, &ifr) == 0) {
+#endif
     found = TRUE;
 
-    if (string_from_sockaddr (&ifr.ifr_addr, buffer, sizeof (buffer)) == 0)
+    if (string_from_sockaddr ((struct sockaddr *)&ifr.CNAME(ifr_addr), buffer, sizeof (buffer)) == 0)
       addr = PyString_FromString (buffer);
   }
 #endif
 
 #if HAVE_SIOCGIFNETMASK
+#if HAVE_SIOCGLIFNUM
+  if (ioctl (sock, SIOCGLIFNETMASK, &ifr) == 0) {
+#else
   if (ioctl (sock, SIOCGIFNETMASK, &ifr) == 0) {
+#endif
     found = TRUE;
 
-    if (string_from_sockaddr (&ifr.ifr_addr, buffer, sizeof (buffer)) == 0)
+    if (string_from_sockaddr ((struct sockaddr *)&ifr.CNAME(ifr_addr), buffer, sizeof (buffer)) == 0)
       netmask = PyString_FromString (buffer);
   }
 #endif
 
 #if HAVE_SIOCGIFFLAGS
+#if HAVE_SIOCGLIFNUM
+  if (ioctl (sock, SIOCGLIFFLAGS, &ifr) == 0) {
+#else
   if (ioctl (sock, SIOCGIFFLAGS, &ifr) == 0) {
+#endif
     found = TRUE;
 
-    if (ifr.ifr_flags & IFF_POINTOPOINT)
+    if (ifr.CNAME(ifr_flags) & IFF_POINTOPOINT)
       is_p2p = TRUE;
   }
 #endif
 
 #if HAVE_SIOCGIFBRDADDR
+#if HAVE_SIOCGLIFNUM
+  if (!is_p2p && ioctl (sock, SIOCGLIFBRDADDR, &ifr) == 0) {
+#else
   if (!is_p2p && ioctl (sock, SIOCGIFBRDADDR, &ifr) == 0) {
+#endif
     found = TRUE;
 
-    if (string_from_sockaddr (&ifr.ifr_addr, buffer, sizeof (buffer)) == 0)
+    if (string_from_sockaddr ((struct sockaddr *)&ifr.CNAME(ifr_addr), buffer, sizeof (buffer)) == 0)
       braddr = PyString_FromString (buffer);
   }
 #endif
 
 #if HAVE_SIOCGIFDSTADDR
+#if HAVE_SIOCGLIFNUM
+  if (is_p2p && ioctl (sock, SIOCGLIFBRDADDR, &ifr) == 0) {
+#else
   if (is_p2p && ioctl (sock, SIOCGIFBRDADDR, &ifr) == 0) {
+#endif
     found = TRUE;
 
-    if (string_from_sockaddr (&ifr.ifr_addr, buffer, sizeof (buffer)) == 0)
+    if (string_from_sockaddr ((struct sockaddr *)&ifr.CNAME(ifr_addr), buffer, sizeof (buffer)) == 0)
       dstaddr = PyString_FromString (buffer);
   }
 #endif
@@ -795,7 +832,7 @@ interfaces (PyObject *self)
 #elif HAVE_SIOCGIFCONF
   const char *prev_name = NULL;
   int fd = socket (AF_INET, SOCK_DGRAM, 0);
-  struct ifconf ifc;
+  struct CNAME(ifconf) ifc;
   int len = -1, n;
 
   if (fd < 0) {
@@ -807,46 +844,59 @@ interfaces (PyObject *self)
 #if HAVE_SIOCGSIZIFCONF
   if (ioctl (fd, SIOCGSIZIFCONF, &len) < 0)
     len = -1;
+#elif HAVE_SIOCGLIFNUM
+  { struct lifnum lifn;
+    lifn.lifn_family = AF_UNSPEC;
+    lifn.lifn_flags = LIFC_NOXMIT | LIFC_TEMPORARY | LIFC_ALLZONES;
+    ifc.lifc_family = AF_UNSPEC;
+    ifc.lifc_flags = LIFC_NOXMIT | LIFC_TEMPORARY | LIFC_ALLZONES;
+    if (ioctl (fd, SIOCGLIFNUM, (char *)&lifn) < 0)
+      len = -1;
+    else
+      len = lifn.lifn_count;
+  }
 #endif
 
   // As a last resort, guess
   if (len < 0)
     len = 64;
 
-  ifc.ifc_len = len * sizeof (struct ifreq);
-  ifc.ifc_buf = malloc (ifc.ifc_len);
+  ifc.CNAME(ifc_len) = len * sizeof (struct CNAME(ifreq));
+  ifc.CNAME(ifc_buf) = malloc (ifc.CNAME(ifc_len));
 
-  if (!ifc.ifc_buf) {
+  if (!ifc.CNAME(ifc_buf)) {
     PyErr_SetString (PyExc_MemoryError, "Not enough memory");
     close (fd);
     return NULL;
   }
 
+#if HAVE_SIOCGLIFNUM
+  if (ioctl (fd, SIOCGLIFCONF, &ifc) < 0) {
+#else
   if (ioctl (fd, SIOCGIFCONF, &ifc) < 0) {
-    free (ifc.ifc_req);
+#endif
+    free (ifc.CNAME(ifc_req));
     PyErr_SetFromErrno (PyExc_OSError);
     close (fd);
     return NULL;
   }
 
   result = PyList_New (0);
-  struct ifreq *pfreq = ifc.ifc_req;
-  for (n = 0; n < ifc.ifc_len;
-       pfreq = (struct ifreq *)((caddr_t)pfreq + len), n += len) {
-    len = sizeof (pfreq->ifr_name) + pfreq->ifr_addr.sa_len;
-
-    if (!prev_name || strncmp (prev_name, pfreq->ifr_name, IFNAMSIZ) != 0) {
-      PyObject *name = PyString_FromString (pfreq->ifr_name);
+  struct CNAME(ifreq) *pfreq = ifc.CNAME(ifc_req);
+  for (n = 0; n < ifc.CNAME(ifc_len)/sizeof(struct CNAME(ifreq));
+      n++,pfreq++) {
+    if (!prev_name || strncmp (prev_name, pfreq->CNAME(ifr_name), IFNAMSIZ) != 0) {
+      PyObject *name = PyString_FromString (pfreq->CNAME(ifr_name));
 
       if (!PySequence_Contains (result, name))
         PyList_Append (result, name);
       Py_XDECREF (name);
 
-      prev_name = pfreq->ifr_name;
+      prev_name = pfreq->CNAME(ifr_name);
     }
   }
 
-  free (ifc.ifc_buf);
+  free (ifc.CNAME(ifc_buf));
   close (fd);
 #endif /* HAVE_SIOCGIFCONF */
 
